package gov.usgs.earthquake.nshmp.netcdf.reader;

import static org.junit.jupiter.api.Assertions.assertArrayEquals;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import org.junit.jupiter.api.Test;

import gov.usgs.earthquake.nshmp.geo.BorderType;
import gov.usgs.earthquake.nshmp.geo.Regions;
import gov.usgs.earthquake.nshmp.netcdf.NshmNetcdfReaderTests;

class NetcdfCoordinatesTest {

  @Test
  final void coordinatesTests() {
    var coords = NshmNetcdfReaderTests.NETCDF.coordinates();

    // Check locations
    assertArrayEquals(NshmNetcdfReaderTests.EXPECTED_LATITUDES, coords.latitudes());
    assertArrayEquals(NshmNetcdfReaderTests.EXPECTED_LONGITUDES, coords.longitudes());

    // Check IMTs
    assertEquals(NshmNetcdfReaderTests.IMTS, coords.imts());
    assertEquals(NshmNetcdfReaderTests.IMTS.size(), coords.imts().size());

    // Check site classes
    assertEquals(NshmNetcdfReaderTests.SITE_CLASSES, coords.siteClasses());

    // Check region
    var border = NetcdfUtils.buildBorder(
        NshmNetcdfReaderTests.EXPECTED_LONGITUDES,
        NshmNetcdfReaderTests.EXPECTED_LATITUDES);
    var expectedRegion = Regions.create("test-region", border, BorderType.MERCATOR_LINEAR);
    var expectedBorder = expectedRegion.border();

    var actualRegion = coords.region();
    var actualBorder = actualRegion.border();

    assertEquals(expectedBorder.size(), actualBorder.size());
    assertEquals(expectedBorder.bounds().max, actualBorder.bounds().max);
    assertEquals(expectedBorder.bounds().min, actualBorder.bounds().min);

    for (var i = 0; i < expectedBorder.size(); i++) {
      assertEquals(expectedBorder.get(i), actualBorder.get(i));
    }

    // Check IMLs
    var actualImls = coords.imls();
    assertEquals(NshmNetcdfReaderTests.IMLS.size(), actualImls.size());

    for (var imlEntry : NshmNetcdfReaderTests.IMLS.entrySet()) {
      var siteClass = imlEntry.getKey();
      assertTrue(actualImls.containsKey(siteClass));
      assertEquals(imlEntry.getValue().size(), actualImls.get(siteClass).size());

      for (var siteEntry : imlEntry.getValue().entrySet()) {
        var imt = siteEntry.getKey();
        var expectedValue = siteEntry.getValue();
        var actualValue = actualImls.get(siteClass).get(imt);

        assertTrue(actualImls.get(siteClass).containsKey(imt));
        assertArrayEquals(expectedValue, actualValue, NshmNetcdfReaderTests.IML_TOL);
      }
    }
  }

}
